@@ -1,7 +1,5 @@
 #!/usr/bin/python3
 #
-# aws2-wrap [-h] [--export] [--profile PROFILE] [--exec <command>] <command>
-#
 # A simple script that exports the accessKeyId, secretAccessKey and sessionToken for the specified
 # AWS SSO credentials, or it can run a subprocess with those credentials.
 #
@@ -17,6 +15,8 @@ import configparser
 import json
 import os
 import pathlib
+import psutil
+import re
 import subprocess
 import sys
 from datetime import datetime, timezone
@@ -89,12 +89,13 @@ def retrieve_token_from_file(filename, sso_start_url, sso_region):
             blob["region"] != sso_region):
         return None
     expires_at = blob["expiresAt"]
-    # This will be a string like "2020-03-26T13:28:35UTC" OR "2021-01-21T23:30:56Z".
+    # This will be a string like "2020-03-26T13:28:35UTC" OR "2021-01-21T23:30:56Z" OR "2021-02-18T18:13:41.632177Z".
     if expires_at[-1] == "Z":
         # Unfortunately, Python version 3.6 or earlier doesn't seem to recognise "Z" so we replace
         # that with UTC first.
         expires_at = expires_at[:-1] + "UTC"
-    expire_datetime = datetime.strptime(expires_at.replace("UTC", "+0000"), "%Y-%m-%dT%H:%M:%S%z")
+    datetime_format = "%Y-%m-%dT%H:%M:%S.%f%z" if "." in expires_at else "%Y-%m-%dT%H:%M:%S%z"
+    expire_datetime = datetime.strptime(expires_at.replace("UTC", "+0000"), datetime_format)
     if expire_datetime < datetime.now(timezone.utc):
         # This has expired
         return None
@@ -236,6 +237,56 @@ def process_cred_generation(
     print("The credentials will expire at %s" % expiration)
 
 
+def run_command(access_key, secret_access_key, session_token, profile, args):
+    """ Run the specified command with the credentials set up """
+    os.environ["AWS_ACCESS_KEY_ID"] = access_key
+    os.environ["AWS_SECRET_ACCESS_KEY"] = secret_access_key
+    os.environ["AWS_SESSION_TOKEN"] = session_token
+    status = None # ensure this is initialised
+    # If region is specified in profile, also set AWS_DEFAULT_REGION
+    if "AWS_DEFAULT_REGION" not in os.environ and "region" in profile:
+        os.environ["AWS_DEFAULT_REGION"] = retrieve_attribute(profile, "region")
+    if args.exec is not None:
+        status = os.system(args.exec)
+    elif args.command is not None:
+        status = os.system(" ".join(args.command))
+    # The return value of os.system is not simply the exit code of the process
+    # see: https://mail.python.org/pipermail/python-list/2003-May/207712.html
+    # noinspection PyUnboundLocalVariable
+    if status is None:
+        sys.exit(0)
+    # noinspection PyUnboundLocalVariable
+    if status % 256 == 0:
+        sys.exit(status//256)
+    sys.exit(status % 256)
+
+
+def export_credentials(access_key, secret_access_key, session_token, profile):
+    """ Export the AWS credentials to environment variables """
+    # On Windows, parent process is aws2-wrap.exe, in unix it's the shell
+    if os.name == "nt":
+        shell_name = psutil.Process().parent().parent().name()
+    else:
+        shell_name = psutil.Process().parent().name()
+
+    is_powershell = bool(re.fullmatch('pwsh|pwsh.exe|powershell.exe', shell_name))
+
+    if is_powershell:
+        print("$ENV:AWS_ACCESS_KEY_ID=\"%s\"" % access_key)
+        print("$ENV:AWS_SECRET_ACCESS_KEY=\"%s\"" % secret_access_key)
+        print("$ENV:AWS_SESSION_TOKEN=\"%s\"" % session_token)
+        # If region is specified in profile, also export AWS_DEFAULT_REGION
+        if "AWS_DEFAULT_REGION" not in os.environ and "region" in profile:
+            print("$ENV:AWS_DEFAULT_REGION=\"%s\"" % retrieve_attribute(profile, "region"))
+    else:
+        print("export AWS_ACCESS_KEY_ID=%s" % access_key)
+        print("export AWS_SECRET_ACCESS_KEY=%s" % secret_access_key)
+        print("export AWS_SESSION_TOKEN=%s" % session_token)
+        # If region is specified in profile, also export AWS_DEFAULT_REGION
+        if "AWS_DEFAULT_REGION" not in os.environ and "region" in profile:
+            print("export AWS_DEFAULT_REGION=%s" % retrieve_attribute(profile, "region"))
+
+
 def main():
     """ Main! """
     args = process_arguments()
@@ -255,12 +306,8 @@ def main():
     session_token = grc_structure["roleCredentials"]["sessionToken"]
     expiration = grc_structure["roleCredentials"]["expiration"]
     if args.export:
-        print("export AWS_ACCESS_KEY_ID=%s" % access_key)
-        print("export AWS_SECRET_ACCESS_KEY=%s" % secret_access_key)
-        print("export AWS_SESSION_TOKEN=%s" % session_token)
-        # If region is specified in profile, also export AWS_DEFAULT_REGION
-        if "AWS_DEFAULT_REGION" not in os.environ and "region" in profile:
-            print("export AWS_DEFAULT_REGION=%s" % retrieve_attribute(profile, "region"))
+        # On Windows, parent process is aws2-wrap.exe, in unix it's the shell
+        export_credentials(access_key, secret_access_key, session_token, profile)
     elif args.generate:
         if args.outprofile is not None:
             process_cred_generation(
@@ -276,26 +323,7 @@ def main():
         }
         print(json.dumps(output))
     else:
-        os.environ["AWS_ACCESS_KEY_ID"] = access_key
-        os.environ["AWS_SECRET_ACCESS_KEY"] = secret_access_key
-        os.environ["AWS_SESSION_TOKEN"] = session_token
-        status = None # ensure this is initialised
-        # If region is specified in profile, also set AWS_DEFAULT_REGION
-        if "AWS_DEFAULT_REGION" not in os.environ and "region" in profile:
-            os.environ["AWS_DEFAULT_REGION"] = retrieve_attribute(profile, "region")
-        if args.exec is not None:
-            status = os.system(args.exec)
-        elif args.command is not None:
-            status = os.system(" ".join(args.command))
-        # The return value of os.system is not simply the exit code of the process
-        # see: https://mail.python.org/pipermail/python-list/2003-May/207712.html
-        # noinspection PyUnboundLocalVariable
-        if status is None:
-            sys.exit(0)
-        # noinspection PyUnboundLocalVariable
-        if status % 256 == 0:
-            sys.exit(status//256)
-        sys.exit(status % 256)
+        run_command(access_key, secret_access_key, session_token, profile, args)
 
 
 if __name__ == '__main__':
